/****************************
 * IBM Confidential
 * Licensed Materials - Property of IBM
 *
 * IBM Rational Developer Traveler
 *
 * (C) Copyright IBM Corporation 2013.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets, 
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */
/*******************************************************************************
 * @license
 * Copyright (c) 2011 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0 
 * (http://www.eclipse.org/legal/epl-v10.html), and the Eclipse Distribution 
 * License v1.0 (http://www.eclipse.org/org/documents/edl-v10.html). 
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
/*global define */

define(['orion/editor/rulers'], function(Ruler) {
	
	/**
	 * Constructs a new sequence numbering ruler. 
	 *
	 * @param {orion.editor.AnnotationModel} annotationModel the annotation model for the ruler.
	 * @param {String} [rulerLocation="left"] the location for the ruler.
	 * @param {orion.editor.Style} [rulerStyle=undefined] the style for the ruler.
	 *
	 * @augments orion.editor.Ruler
	 * @class This objects implements a sequence numbering ruler.
	 *
	 * <p><b>See:</b><br/>
	 * {@link orion.editor.Ruler}
	 * </p>
	 * @name edit.SequenceNumberRuler
	 */
	function SequenceNumberRuler (annotationModel, rulerLocation, rulerStyle, sequenceNumbers, sequenceNumberSpec) {
		Ruler.Ruler.call(this, annotationModel, rulerLocation, "page", rulerStyle); //$NON-NLS-0$
		this._sequenceNumbers = sequenceNumbers;
		this._sequenceNumberSpec = sequenceNumberSpec;
		this._numDigits = this._sequenceNumberSpec.end - this._sequenceNumberSpec.start;
	}
	SequenceNumberRuler.prototype = new Ruler.Ruler(); 
	/** @ignore */
	SequenceNumberRuler.prototype.getAnnotations = function(startLine, endLine) {
		var result = Ruler.Ruler.prototype.getAnnotations.call(this, startLine, endLine);
		var model = this._view.getModel();
		for (var lineIndex = startLine; lineIndex < endLine; lineIndex++) {
			var mapLine = lineIndex;
			if (model.getBaseModel) {
				var lineStart = model.getLineStart(mapLine);
				mapLine = model.getBaseModel().getLineAtOffset(model.mapOffset(lineStart));
			}
			if(this._sequenceNumbers[mapLine]){
				if (!result[lineIndex]) { result[lineIndex] = {}; }
				result[lineIndex].html = this._sequenceNumbers[mapLine].replace(/\s/g, '&nbsp;');
			}
		}
		return result;
	};
	/** @ignore */
	SequenceNumberRuler.prototype.getWidestAnnotation = function() {
		return {"html":(new Array(this._numDigits)).join("&nbsp")};
	};
	/** @ignore */
	SequenceNumberRuler.prototype._onTextModelChanged = function(e) {
//		var start = e.start;
//		var model = this._view.getModel();
//		if(e.removedLineCount > 0 || e.addedLineCount > 0){
//			var startLine = model.getLineAtOffset(start) + 1;
//			var newSequenceNumbers = [];
//			for(var i = 0; i < e.addedLineCount; i++){
//				newSequenceNumbers.push("");
//			}
//			[].splice.apply(this._sequenceNumbers, [startLine, e.removedLineCount].concat(newSequenceNumbers));
//		}		
	};
	
	return {
		SequenceNumberRuler: SequenceNumberRuler
	};
});
