/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide( "com.ibm.teampdp.query.engine.web.ui.internal.page.RppJazzPage" );

dojo.require( "dijit._Widget" );
dojo.require( "dijit._WidgetBase" );
dojo.require( "dijit._TemplatedMixin" );
dojo.require( "dijit._WidgetsInTemplateMixin" );
dojo.require( "net.jazz.ajax.ui.PlatformUI" );
dojo.require( "dojo.i18n" );

// Dojo Widgets in rppClient.html
dojo.require( "dijit.layout.TabContainer" );
dojo.require( "dijit.layout.ContentPane" );
dojo.require( "dijit.form.TextBox" );
dojo.require( "dijit.form.CheckBox" );
dojo.require( "dojox.widget.Standby" );
dojo.require( "com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/widgets/RPPTabController");

dojo.requireLocalization( "com.ibm.teampdp.query.engine.web.ui.internal.page", "searchPage" );

var rppUseInlineCodeEdit = false;
var rppDebug = false;

// global variables
var rppDefaultLabelWidth = 250;
var rppDefaultTextWidth  = "300px";

var rppCurrentPaneHeight = -1;
var rppCurrentPaneWidth = -1;
var rppCurrentPaneGridHeight = -1;

( function () {

	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	dojo.declare( "com.ibm.teampdp.query.engine.web.ui.internal.page.RppJazzPage", [
			dijit._Widget, dijit._WidgetBase, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], {
		
		
		templatePath : dojo.moduleUrl( "com.ibm.teampdp.query.engine.web", "ui/internal/page/rppClient.html" ),

		constructor : function () {
			this.page = dojo.getLocalization( "com.ibm.teampdp.query.engine.web.ui.internal.page", "searchPage" );
		},

		postCreate : function () {
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction( "com.ibm.teampdp.query.engine.web.rpp", this, "rpp" );
			require( ["dojo/dom-construct", "dojo/cookie",
				"com.ibm.teampdp.query.engine.web/ui/internal/page/typeEnable",
				"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
		], function (domConstruct, cookie, typeEnable, typeDict ) {
			var searchForItem = "";
			var nlsTypes = [];
			typeEnable.forEach(function(type) {
				var nlsType = typeDict[type];
				if ( nlsType == undefined )
					nlsType = type + " (not in nls typeDict!!)";
				nlsTypes.push([nlsType,type]);
			});
			nlsTypes.sort();
			nlsTypes.forEach(function(type) {
				searchForItem = searchForItem + 
				"<div class='search_for_item'>"+
				"<input id='"+type[1]+"' class='search_for' name='searchFor'"+
				"	type='checkbox' value='"+type[1]+"' onChange='enableSearchButton()' />&nbsp;"+
				"<span class='display_small_icon  folder'></span>&nbsp;&nbsp;&nbsp;"+type[0]+
				//"<label for='"+type[1]+"'>&nbsp;&nbsp;&nbsp;"+type[0]+"</label>"+
				"</div>"
			});
			
			var row = domConstruct.toDom(searchForItem);
		    domConstruct.place(row, "rppEnabledTypes");

			var _rppDebug = qs("rppDebug");

			if ( _rppDebug != undefined && "true" == _rppDebug ) {
				rppDebug = true;
			} 
				
			if ( rppDebug == true ) {
				var rppCookie = cookie("rppOpenLastModel");
				if ( rppCookie != undefined ) {
					var row = domConstruct.toDom("<div><button type='button' class='button-item' onClick='rppOpenLastModel()'><b>Open Last Opened Model</b></button></div>");
					domConstruct.place(row, "rppOpenLastModel");
				}
				var rppGenCookie = cookie("rppOpenLastGenerated");
				if ( rppGenCookie != undefined ) {
					var row = domConstruct.toDom(
							"<div><button type='button' class='button-item' onClick='rppOpenLastGenerated()'><b>Open Last Opened Generated</b></button></div>");
					domConstruct.place(row, "rppOpenLastGenerated");
				}
			}
		} );
		},

		rpp : function () {
			require( [
					"dijit/registry", "dojo/request/xhr", "dojo/dom", "dojo/on", 'dojox.timing',
					"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/widgets/RadicalEntityWidget",
					"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/widgets/OrionViewerWidget",
			], function ( registry, xhr, dom, on, timing, RadicalEntityWidget, OrionViewerWidget ) {
				xhr( net.jazz.ajax.contextRoot() + "/service/com.ibm.teampdp.query.engine.common.IRppQueryEngineService", {
					method : "POST",
					handleAs : "json",
					query : {
						search : "searchStreams",
						projectAreaId : jazz.app.context.get().itemId,
					}
				} ).then(
						function ( data ) {
							require( [
									"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/createStreamSelect",
									"com.ibm.teampdp.query.engine.web/ui/internal/page/maxNumber", "dojo/dom-construct",
							], function ( streamSelect, maxNumber, domConstruct ) {
								streamSelect.createStreamSelect( data ).placeAt( "serverScope" );
								domConstruct.create( "input", {
									type : "hidden",
									name : "maxNumber",
									value : maxNumber.MAX_NUMBER_SEARCH,
								}, "searchForm" );
								var expression = registry.byId("expression");
							    on(expression, "change", function(){
							    	enableSearchButton();
							    });
							    if ( rppDebug == true ) {
							    	expression.setValue('*');
							    }
								enableSearchButton();
								var cP = registry.byId( "rppSearchContentPane" );
								cP.set("iconClass", "display_icon search_view");
								
								setContentPaneSize(cP);
								var inResize = false;
								dojo.connect(registry.byId( "tabContainer" ), "resize", function(evt){
									if ( inResize == false ) {
										inResize = true;
										rppCurrentPaneHeight = -1;
										var tc = registry.byId( "tabContainer" );
										tc.getChildren().forEach( function ( item ) {
											setContentPaneSize(item);
										});
										var currentPane = tc.selectedChildWidget;
										if ( currentPane != undefined ) {
											tc.selectChild(cP);
											tc.selectChild(currentPane);
										}
										/*tc.selectedChildWidget.getChildren().forEach( function ( item ) {
											item.getChildren().forEach( function ( item ) {
												if ( item.declaredClass && item.declaredClass == "dijit.layout.TabContainer") {
													item.getChildren().forEach( function ( item ) {
														console.debug(item);
														//setContentPaneSize(item);
													});
												}
											});
										});*/
										inResize = false;
									}
								});				
								
							} );
						}, function ( err ) {
							displayRppError( err );
						} );
			} );
		}
	} );
} )();

// Set the size of the pane
function setContentPaneSize(cpPane) {
	if ( rppCurrentPaneHeight == -1 ) {
		var w = window,
		d = document,
		e = d.documentElement,
		g = d.getElementsByTagName('body')[0],
		x = w.innerWidth || e.clientWidth || g.clientWidth,
		y = w.innerHeight|| e.clientHeight|| g.clientHeight;
	
		var _rppCurrentPaneHeight = y - 190;
		
		if ( _rppCurrentPaneHeight == rppCurrentPaneHeight  )
			return false;
		rppCurrentPaneHeight = _rppCurrentPaneHeight;
		rppCurrentPaneWidth = x - 100;
		rppCurrentPaneGridHeight = rppCurrentPaneHeight - 150;
	}

	cpPane.domNode.style.height = rppCurrentPaneHeight +'px';
	
	if ( cpPane.id == "rppSearchContentPane" ) {
		require( ["dojo/dom"], function ( dom ) {
				var rppEnabledTypes = dom.byId("rppEnabledTypes");
				rppEnabledTypes.style.height = ( rppCurrentPaneHeight - 170 ) +'px';
	
				var rppEnabledTypes = dom.byId("searchIn");
				rppEnabledTypes.style.height = ( rppCurrentPaneHeight - 170 ) +'px';
		} );
	}
	
	return true;
};	


function qs(version) {
	var query = window.location.href;
	var parms = query.split('&');
	for (var i=0; i<parms.length; i++) {
		var pos = parms[i].indexOf('=');
		if (pos > 0  && version == parms[i].substring(0,pos)) {
			return parms[i].substring(pos+1);;
		}
	}
	return "";
};	

// for the checkbox normal => searchFor
function checkAll ( checkboxname, checktoggle ) {
	dojo.query( 'INPUT[name="' + checkboxname + '"]' ).forEach( function ( checkbox ) {
		if ( checkbox.type == "checkbox" )
			checkbox.checked = checktoggle;
	} );
	enableSearchButton();
}

// for the checkbox widget => searchFor
function checkWidgetAll ( checkboxname, checktoggle ) {
	dojo.query( 'INPUT[name="' + checkboxname + '"]' ).forEach( function ( checkbox ) {
		var node = dijit.getEnclosingWidget( checkbox );
		if ( node.type == "checkbox" )
			node.set( 'checked', checktoggle );
	} );
}

function checkForm () {
	// Check the form
	if ( dojo.query( 'INPUT[name=searchFor]:checked', 'searchForm' ).length == 0 ) {
		displayRppWarningDialog( "Please chose the <Search For>" );
		return false;
	}
	return true;
}

function displayRppError (err) {
	require( [
		"dijit/registry"
		], function ( registry ) {
			var standby = registry.byId( "rppStandby" );
			standby.hide();
	} );
	console.debug("ERROR : " + err);
	if ( err.response ) {
		console.debug("RESPONSE : ");
		console.debug(err.response);
	}
	if ( err && err.response && err.response.xhr && err.response.xhr.getResponseHeader ) {
		var headerValue = err.response.xhr.getResponseHeader("X-com-ibm-team-repository-web-auth-msg");
		if ( headerValue == "authrequired" ) {
			window.location.reload(true);
			return;
		} else if ( err.response.text ) {
			if ( err.response.text.indexOf('com.ibm.teampdp.web.ui.readaccess') >= 0 
						|| err.response.text.indexOf('com.ibm.team.scm.readaccess.general') >= 0 ) {
				require( [
					"dojo/dom-construct", "dijit/registry"
					], function ( domConstruct, registry ) {
						var tc = registry.byId( "tabContainer" );
						var node = tc.domNode.parentNode;
						tc.destroy();
					    node.innerHTML="<p><font color='red'><b>"+err.response.text.replace('\n','<p></p>')+"</b></font></p>";
				} );
			} else {
				require( [
					"dojo/json"
					], function ( JSON ) {
						try {
							var data = JSON.parse( err.response.text );
							if ( data.errorTrace ) {
								var message = "";
								if ( data.errorMessage )
									message += "Message : "+data.errorMessage;
								if ( data.errorClass )
									message += "<br>Class : "+data.errorClass;
								if ( data.errorCode )
									message += "<br>Code : "+data.errorCode;
								message += "<br>Trace : ";
								data.errorTrace.forEach(function(trace) {
									message += "<br>"+trace;
								});
								displayRppErrorDialog(  message  );
							} else {
								displayRppErrorDialog(  err.response.text  );
							}
						} catch(e){
							displayRppErrorDialog(  err.response.text  );
						}
				} );
			}
			return;
		}
	} else {
		if ( err && err.response && err.response.xhr && err.response.xhr.responseURL ) {
			var responseUrl = err.response.xhr.responseURL;
			if ( responseUrl.endsWith("authrequired") ) {
				window.location.reload(true);
				return;
			}
		}
	}	
	displayRppErrorDialog( "A problem occurred : " + err);
}

function displayRppErrorDialog (err) {
	require( [
		"dijit/Dialog",
		], function ( Dialog ) {
	    	dlg = new dijit.Dialog({
	    		title:'Fatal Error',
	    		content:'<div>'+err+'</div>',
	    		style:"width:1000px;height:500px;overflow:auto"
	    	});
	    dlg.show();
	} );
}

function displayRppWarningDialog (err) {
	require( [
		"dijit/Dialog",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PTEditor"
		], function ( Dialog, PTEditor ) {
	    	dlg = new dijit.Dialog({
	    		title: PTEditor.WARNING,
	    		content:'<div>'+err+'</div>',
	    		style:"overflow:auto"
	    	});
	    dlg.show();
	} );
}


function rppEnumVDislayValue (enumPage, value) {
	if ( value == undefined || value[ "class" ] == undefined || value[ "value" ] == undefined)
		return value;
	return enumPage[ value[ "class" ] + value[ "value" ].toUpperCase() ];
}

function rppScreenEnumVDislayValue (data, enumPage, valueAttribute, inheritedMessage) {
	if ( valueAttribute == undefined )
		return "N/A";
	var value = data[valueAttribute];
	var result = value[ "value" ].toUpperCase();

	if ( data[ "class" ] != "PacDialog" && ( result == "_NONE"  || result == "_INHERITED") ) {
		value = data.dialog[valueAttribute];
		if ( value == undefined )
			return "N/A";
		return enumPage[ value[ "class" ] + value[ "value" ].toUpperCase() ] + " ("+inheritedMessage+")";
	}
	
	return enumPage[ value[ "class" ] + value[ "value" ].toUpperCase() ];
}

function rppFolderEnumVDisplayValue (data, enumPage, valueAttribute, inheritedMessage) {
	if ( valueAttribute == undefined )
		return "N/A";
	var value = data[valueAttribute];
	var result = value[ "value" ].toUpperCase();

	if ( data[ "class" ] != "PacDialogFolder" && ( result == "_NONE"  || result == "_INHERITED") ) {
		value = data.pacDialogFolder[valueAttribute];
		if ( value == undefined )
			return "N/A";
		return enumPage[ value[ "class" ] + value[ "value" ].toUpperCase() ] + " ("+inheritedMessage+")";
	}

	return enumPage[ value[ "class" ] + value[ "value" ].toUpperCase() ];
}

function enableSearchButton () {
	require( [
			"dojo/dom-attr", "dijit/registry"
	], function ( domAttr, registry ) {
		if ( registry.byId( "streamId" ) != undefined && dojo.query( 'INPUT[name=searchFor]:checked', 'searchForm' ).length > 0
				&& dojo.query( 'INPUT[name=searchIn]:checked', 'searchForm' ).length > 0 
				&& registry.byId("expression").getValue().length > 0 ) {
			domAttr.remove( "search", "disabled" );
		} else {
			domAttr.set( "search", "disabled", "disabled" )
		}
	} );
}

( function sendSearchForm () {
	var searchForm = dojo.byId( "searchForm" );
	dojo.connect( searchForm, "onsubmit", function ( event ) {
		// Stop the submit event since we want to control form submission.
		dojo.stopEvent( event );
		// check the form
		if ( !checkForm() ) {
			return;
		}
		require( [
				"dijit/registry", "dijit/layout/TabContainer", "dojo/dom-form",
				"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/createSearchPane"
		], function ( registry, TabContainer, domForm, createSearchPane ) {
			var standby = registry.byId( "rppStandby" );
			standby.show();
			// check whether all projects are selected
			// if all checked, don't send projects to server
			var searchInCkeckBox = dojo.query( 'INPUT[name=searchIn]', 'searchForm' );
			if ( dojo.query( 'INPUT[name=searchIn]:checked', 'searchForm' ).length == searchInCkeckBox.length ) {
				searchInCkeckBox.attr( "disabled", true );
			}
			// get the form variables
			var formJson = JSON.parse( domForm.toJson( "searchForm" ) );
			searchInCkeckBox.attr( "disabled", false );
			// get streamId and stream name
			var streamId = registry.byId( "streamId" ).getValue();
			var streamName = registry.byId( "streamId" ).attr( 'displayedValue' );
			var hasCpSearch = registry.byId( "cpSearch" ) != undefined;
			// get search pane
			var cpSearch = createSearchPane.createPane( formJson, streamId, streamName );

			// create the table container
			var tc = registry.byId( "tabContainer" );
			if ( hasCpSearch == false )
				tc.addChild( cpSearch );
			tc.selectChild( cpSearch );
		} );
	} );
} )();


function rppOpenLastModel ( ) {
	require( [
		"dojo/cookie"
], function ( cookie ) {
	var items = cookie("rppOpenLastModel").split('/');
	openRadicalElement ( items[0], items[3], items[4], items[5], items[6] )
} );
}

function rppOpenLastGenerated ( ) {
	require( [
		"dojo/cookie"
], function ( cookie ) {
	var items = cookie("rppOpenLastGenerated").split('/');
	openArtifactFile ( items[0], items[1], items[2], items[3], items[3], items[4], null, items[5] );
} );
}

//open the content of design file
function openRadicalElement ( streamId, workingProject, project, type, name ) {
	require( [
				"dojo/request/xhr",
		], function ( xhr ) {
		var formJson = {
				expression : name,
				limitTo : "name",
				maxNumber : 1,
				searchFor : type,
				streamId : streamId,
				search : "searchItems",
				project : workingProject,
				caseSentive	: "on"
			};
		xhr( net.jazz.ajax.contextRoot() + "/service/com.ibm.teampdp.query.engine.common.IRppQueryEngineService", {
			method : "POST",
			data : formJson,
			handleAs : "json"
		} ).then( function ( dataJSONSearch ) {
			var result = dataJSONSearch[ 0 ].result[0];
			openArtifactFile(streamId, result.compentId, result.fileItemId,  workingProject, result.project, result.type, result.metaType, result.name);
		}, function ( err ) {
			displayRppError( err );
		} );
	} );	
}

//open the content of design file
function openArtifactFile ( streamId, compentId, fileItemId, workingProject, project, type, metaType, name) {
	require( [
		"dojo/cookie", "dijit/layout/ContentPane", "dojox/layout/TableContainer", "dijit/registry", "dojox/widget/Standby",
 		"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/createContentPane",
		"com.ibm.teampdp.query.engine.web/ui/internal/page/typeEnable", "com.ibm.teampdp.query.engine.web/ui/internal/page/typeRelation"
	], function ( cookie, ContentPane, TableContainer, registry, Standby, createContentPane, typeEnable, typeRelation ) {
		var typesRelation = typeRelation;
		var typesEnable = typeEnable;
		var isGeneratedArtifact = false;
		if ( workingProject == undefined || type == undefined /*|| typesEnable.indexOf( type ) == -1*/ ) {
			displayRppErrorDialog('Unexpected error in openArtifactFile!!');
				return;
		}
		if ( typesRelation.indexOf( type ) > -1 ) {
			isGeneratedArtifact = true;
		}
		var tabContainer = registry.byId( "tabContainer" );
		var tabId = streamId + "#" + fileItemId;
		var cpContent = registry.byId( tabId );
		if ( cpContent == undefined ) {
			var cpContent = new ContentPane( {
				title : name + (metaType != undefined && metaType.length > 0 ? " (" + metaType + ")" : ""),
				closable : true,
				iconClass : "display_icon loadingAnimation",
				id: tabId,
				style : {
					'overflow' : 'auto',
				},
			} );
			var standby = registry.byId( "rppStandby" );
			standby.show();
			tabContainer.addChild( cpContent )
			if ( isGeneratedArtifact )  {
				setContentPaneSize(cpContent);
				cookie("rppOpenLastGenerated", streamId+"/"+compentId+"/"+fileItemId+"/"+workingProject+"/"+type+"/"+ name, { expires: 5 });
				var sourceViewer = 
					new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.OrionViewerWidget(
							"", "", type, workingProject, streamId, compentId, fileItemId, true, rppCurrentPaneHeight );
				
				cpContent.addChild( sourceViewer );
				cpContent.set( "iconClass", "display_icon " + type );

				tabContainer.selectChild( cpContent );
				
			} else {
				setContentPaneSize(cpContent);
				cookie("rppOpenLastModel", streamId+"/"+compentId+"/"+fileItemId+"/"+workingProject+"/"+project+"/"+type+"/"+ name, { expires: 5 });
				var promiseContent = createContentPane.createPane( streamId, fileItemId, compentId, workingProject, project, type, name );
				promiseContent.then( function ( contentPane ) {			
					cpContent.addChild( contentPane );
					cpContent.set( "iconClass", "display_icon " + type );
					tabContainer.selectChild( cpContent );
				} );
			}
			standby.hide();
		} else {
			setContentPaneSize(cpContent);
			tabContainer.selectChild( cpContent );
		}
	} );
}

function createReferencePane ( name, type, project, context, direction, fileItemId, compentId, streamId ) {
	require( [
			"dijit/registry", "dijit/layout/ContentPane", "com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/createReferenceTree",
			"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/exportButton",
			"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/searchPage"
	], function ( registry, ContentPane, createReferenceTree, exportButton, searchPage ) {
		// get the tabcontainer
		var tabContainer = registry.byId( "tabContainer" );
		// test whether cp is already exist
		var referencePaneId = direction + "/" + name + "/" + type + "/" + project;
		var referencePane = registry.byId( referencePaneId );
		if ( referencePane == undefined ) {
			// construct the reference content pane
			referencePane = new ContentPane( {
				title : searchPage[ direction ] + " " + name,
				closable : true,
				iconClass : "display_icon loadingAnimation",
				id : referencePaneId,
				style : {
					'overflow' : 'auto',
				},
			} );
			tabContainer.addChild( referencePane );
			// add export button
			var formJson = {
				fileName : name,
				anchorName : name,
				type : type,
				project : project,
				context : context,
				direction : direction,
				streamId : streamId,
				search : "searchReferences",
			};
			var buttonExport = exportButton.exportButton( formJson );
			referencePane.addChild( buttonExport );
			// add referencePane
			var promiseReferenceTree = createReferenceTree.createTree( direction, name, type, project, context, fileItemId, compentId, streamId );
			promiseReferenceTree.then( function ( referenceTree ) {
				referencePane.addChild( referenceTree );
				referencePane.set( "iconClass", "display_icon reference_view" );
				tabContainer.selectChild( referencePane );
			} );
		} else {
			tabContainer.selectChild( referencePane );
		}
		setContentPaneSize(referencePane);
	} );
}

// download generated file
function downloadGeneratedFile ( streamId, compentId, fileItemId, name, type ) {
	require( [
		"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/createFileDownload"
	], function ( createFileDownload ) {
		createFileDownload.downloadFile( streamId, compentId, fileItemId, name, type );
	} );
}


// MS don't support String.endWith !!! (support start from MS Edge in windows 10)
if ( typeof String.prototype.endsWith != 'function' ) {
	  String.prototype.endsWith = function( str ) {
	    return this.substring( this.length - str.length, this.length ) === str;
	  }
};