/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/request/xhr", "dijit/registry", "dijit/layout/ContentPane", "dojo/string",
		"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/handleSearchJSON",
		"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/createSearchTree",
		"com.ibm.teampdp.query.engine.web/ui/internal/page/jsModules/exportButton",
		"com.ibm.teampdp.query.engine.web/ui/internal/page/maxNumber",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/searchPage",
], function ( xhr, registry, ContentPane, string, handleSearchJSON, createSearchTree, exportButton, maxNumber, searchPage ) {
	return {
		createPane : function ( formJson, streamId, streamName ) {
			// create the table container
			var cpSearch = registry.byId( "cpSearch" );
			if ( cpSearch == undefined ) {
				var cpSearch = new ContentPane( {
					title : searchPage.SEARCH_TABTEXT,
					id : "cpSearch",
					style : {
						'overflow' : 'auto',
					},
				} );
			}
			setContentPaneSize(cpSearch);
			// Ajax for design file
			cpSearch.set( "iconClass", "display_icon loadingAnimation" );
			xhr( net.jazz.ajax.contextRoot() + "/service/com.ibm.teampdp.query.engine.common.IRppQueryEngineService", {
				// handleAs : "json",
				method : "POST",
				data : formJson,
				handleAs : "json"
			} ).then( function ( dataJSONSearch ) {
				// check whether server cut the response.
				var totalNumbers = dataJSONSearch[ 0 ].number;
				if ( totalNumbers > maxNumber.MAX_NUMBER_SEARCH ) {
					displayRppWarningDialog( string.substitute( searchPage.ALERT_EXCEED, [
							totalNumbers, maxNumber.MAX_NUMBER_SEARCH
					] ) );
				}
				// treat the search item result
				var resultJSONProject = handleSearchJSON.topLevelProject( dataJSONSearch[ 0 ].result, streamId, streamName );
				var treeSearch = createSearchTree.createTree( resultJSONProject, streamId );
				// get the export button
				formJson.fileName = streamName;
				var buttonExport = exportButton.exportButton( formJson );
				// add the export button
				cpSearch.set( "content", "" );
				cpSearch.addChild( buttonExport );
				// add the search pattern
				var searchPatternCP = new ContentPane( {
					content : "<b>"+string.substitute( searchPage.SEARCH_PATTERN, [
							formJson.expression, searchPage[ "_" + formJson.limitTo.toUpperCase() ], totalNumbers
					] ) + "</b>",
				} );
				cpSearch.addChild( searchPatternCP );
				// add search tree
				cpSearch.addChild( treeSearch );
				cpSearch.set( "iconClass", "display_icon search_result" );
				setContentPaneSize(cpSearch);
				var standby = registry.byId( "rppStandby" );
				standby.hide();
			}, function ( err ) {
				displayRppError( err );
			} );
			return cpSearch;
		}
	};
} );