/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dijit/layout/ContentPane", "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( ContentPane, PacbaseEditor, typeDict ) {
	return {
		getOverviewDLines : function ( dLines, type ) {
			var overviewDLines;
			require( [
				"dojo/string"
			], function ( string ) {
				overviewDLines = new ContentPane( {
					content : "<hr><strong>" + PacbaseEditor.CELINE_SECTION_HEADER + ": </strong><P>"
							+ string.substitute( PacbaseEditor.DLINE_SECTION_DESCRIPTION, [
								typeDict[ type ]
							] ) + "</p>",
				} );
			} );
			if ( dLines.length != 0 ) {
				var dLinesContent = "<ul>"
				dLines.forEach( function ( dLine ) {
					dLinesContent += "<li>";
					dLinesContent += dLine[ "lineType" ] + ",";
					dLinesContent += dLine[ "more" ] + ",";
					dLinesContent += dLine[ "allowedValues" ] + ",";
					dLinesContent += dLine[ "description" ];
					dLinesContent += "</li>";
				} );
				dLinesContent += "</ui>";
				var dLinesPane = new ContentPane( {
					content : dLinesContent,
					style : {
						height : "100px"
					}
				} );
				overviewDLines.addChild( dLinesPane );
			}
			return overviewDLines;
		},

		getDetailDLines : function ( dLines, type ) {
			var detailDLines;
			require( [
				"dojo/string"
			], function ( string ) {
				detailDLines = new ContentPane( {
					content : "<strong>" + PacbaseEditor.DLINE_SECTION_HEADER + ": </strong><hr><P>"
							+ string.substitute( PacbaseEditor.DLINE_EDIT_SECTION_DESCRIPTION, [
								typeDict[ type ]
							] ) + "</p>",
				} );
			} );
			require( [
					'dojox/grid/DataGrid', 'dojo/data/ItemFileReadStore'
			], function ( DataGrid, ItemFileReadStore ) {
				/* set up data store */
				var store = new ItemFileReadStore( {
					data : {
						items : dLines
					}
				} );

				/* set up layout */
				var layout = [
						{
							'name' : PacbaseEditor.DLINE_TYPE,
							'field' : 'lineType'
						}, {
							'name' : PacbaseEditor.MORE,
							'field' : 'more'
						}, {
							'name' : PacbaseEditor.ALLOWED_VALUES,
							'field' : 'allowedValues',
							'width' : '150px'
						}, {
							'name' : PacbaseEditor.DLINE_DESCRIPTION,
							'field' : 'description',
							'width' : '400px'
						}
				];

				/* create a new grid */
				var dLinesGrid = new DataGrid( {
					store : store,
					structure : layout,
					autoHeight : true,
					autoWidth : true,
					style : {
						'overflow' : 'auto',
					},	
					onStyleRow: function (row) {
						row.customClasses = "rppGridFixedDefaultFont";
					},
					selectable : true,
					canSort : function () {
						return false;
					}
				} );
				detailDLines.addChild( dLinesGrid );
			} );
			return detailDLines;
		}
	};
} );