/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/string", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/tree/ForestStoreModel",
		"dojo/data/ItemFileReadStore", "dojox/grid/LazyTreeGrid", "dijit/form/Button", "dijit/Dialog", "dijit/Tooltip", "./gLinesPane",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( string, TableContainer, ContentPane, TextBox, ForestStoreModel, ItemFileReadStore, LazyTreeGrid, Button, Dialog, Tooltip, gLinesPane, 
				PacbaseEditor, typeDict ) {	

	
	function getValue ( value ) {		
		return value.toString().replace("_","").replace("None","");
	}

	function getNodecode ( item ) {
		if (item.nodeCode != undefined && item.nodeCode != "" && item["class"] != undefined) { 
			var code = item.nodeCode;

			if (item["class"] == "PacChildNode")  
				return 	"<span class='display_icon pacchildnode'></span>" + code;

			if (item["class"] == "PacRootNode")  
				return 	"<span class='display_icon pacrootnode'></span>" + code;
		}		
		if (item["class"] != undefined && item["class"] == "PacNodeDisplayKey")  
			return 	"<span class='display_icon pacnodedisplaykey'></span> ";

		return "";		
	}
	
	function getPacServer ( item, store, project, streamId ) {
		// item is pacServer
		if (item.pacServer != undefined) {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("",item.pacServer[0], true, streamId, project, item.pacServer[0].name, "",false);			

		}
		return "";
	}

	function getLogicalView ( item, store, project, streamId ) {
		// item is logicalView
		if (item.logicalView != undefined) {
			return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("",item.logicalView[0], true, streamId, project, item.logicalView[0].name, "",false);			

		}
		return "";
	}

	function getTypeNode ( item ) {
		if (item.typeNode != undefined && item.typeNode != "") { 
			return getValue(item.typeNode[0].value);
		}
		return "";		
	}

	function getCardinality ( item ) {
		if (item.cardinality != undefined && item.cardinality != "") { 
			return getValue(item.cardinality[0].value);
		}
		return "";		
	}

	function getSubschema ( item ) {
		if (item.subschema != undefined && item.subschema != "") { 
			return getValue(item.subschema[0].value);
		}
		return "";		
	}

	function getDataElementKey ( item, store, project, streamId  ) {
		
		if (item.dataElementKey != undefined && item.dataElementKey != "" ) { 
			if (  item.dataElementKey[0]["class"] != "dataelement")
				return item.dataElementKey[0].name;
			else
				return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("",item.dataElementKey[0], false, streamId, project, "", "",false);
		}
		return "";		
	}

	function getDataElementSource ( item, store, project, streamId  ) {
		if (item.dataElementSource != undefined && item.dataElementSource != "") { 
			if (  item.dataElementSource[0]["class"] != "dataelement")
				return item.dataElementSource[0].name;
			else
				return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("",item.dataElementSource[0], false, streamId, project, "", "",false);
		}	
		return "";		
	}

	return {
		
		getDetailCSLines : function ( pacRootNode, type, project, streamId ) 
		{	
			var cpDetailCSLines = new ContentPane( {
				content : "<strong>" + PacbaseEditor.FOLDER_SECTION_HEADER + ": </strong><hr><P>"
						+ string.substitute( PacbaseEditor.FOLDER_TABLE_SECTION_DESCRIPTION, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			// prepare lazy tree grid
			var layoutCSLines = [
					{
						name : PacbaseEditor.FOLDER_COLUMN_NODE_CODE,
						width : "160px",
						field : "_item",
						formatter : function ( item ) {
							return getNodecode (item, this.grid.store, project, streamId);
						}
					}, {						
						name : PacbaseEditor.FOLDER_COLUMN_COMPONENT,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {
							return getPacServer (item, this.grid.store, project, streamId);
						} 
					}, {						
						name : PacbaseEditor.FOLDER_COLUMN_LOGICAL_VIEW,
						width : "80px",
						field : "_item",
						formatter : function ( item ) {
							return getLogicalView (item, this.grid.store, project, streamId);
						} 
					}, {						
						name : PacbaseEditor.FOLDER_COLUMN_TYPE_NODE,
						width : "30px",
						field : "_item",
						formatter : function ( item ) {
							return getTypeNode (item, this.grid.store, project, streamId);
						} 
					}, {						
						name : PacbaseEditor.FOLDER_COLUMN_CARDINALITY,
						width : "30px",
						field : "_item",
						formatter : function ( item ) {
							return getCardinality (item, this.grid.store, project, streamId);
						} 
					}, {						
						name : PacbaseEditor.FOLDER_COLUMN_SUBSCHEMA,
						width : "70px",
						field : "_item",
						formatter : function ( item ) {
							return getSubschema (item, this.grid.store, project, streamId);
						} 
					}, {						
						name : PacbaseEditor.FOLDER_COLUMN_DATAELEMENT_SOURCE,
						width : "60px",
						field : "_item",
						formatter : function ( item ) {
							return getDataElementSource (item, this.grid.store, project, streamId);
						} 
					}, {						
						name : PacbaseEditor.FOLDER_COLUMN_DATAELEMENT_KEY,
						width : "60px",
						field : "_item",
						formatter : function ( item ) {
							return getDataElementKey (item, this.grid.store, project, streamId);
						} 
					}				 
			];			

			// noeud racine vide
			var pacRootNodes = [];
			
			if (pacRootNode != undefined && pacRootNode != "") {
				pacRootNodes = [pacRootNode];
			} 

			// prepare lazy tree grid
			var data = {
				items : pacRootNodes
			};
			var store = new ItemFileReadStore( {
				data : data,
			} );

			
			var index = 0;
			data.items.forEach(function(item) {
				item.treeId = index++;
			});
			
			
			var model = new ForestStoreModel( {
				store : store,
				mayHaveChildren : function ( item ) {
					if ( item.childNodes != undefined && item.childNodes.length > 0 ) {						
						return true;
					}
					if (item.displayKeys != undefined && item.displayKeys.length > 0) {
						return true;
					}		
					return false;
				},
				getChildren : function ( item, onComplete, onError ) {
					var childKeys = null;
					var childNodes = null;
					if ( item.childNodes != undefined && item.childNodes.length > 0) {
						childNodes = item.childNodes;
					}
					if (item.displayKeys != undefined && item.displayKeys.length > 0) {
						childKeys = item.displayKeys;
					}	
					if ( childNodes != null ) {
						index = 0;
						childNodes.forEach(function(child) {
							child.treeId = item.treeId + index;
						});
						onComplete(childNodes);
					}
					if ( childKeys != null ) {
						index = 0;
						childKeys.forEach(function(child) {
							child.treeId = item.treeId + index;
						});
						onComplete(childKeys);
					}
				}
			} );
		
			var hearderTooltips =  [PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_NODE_CODE, 
			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_COMPONENT,
			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_LOGICAL_VIEW,
			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_TYPE_NODE,
			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_CARDINALITY,
			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_SUBSCHEMA,
			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_DATAELEMENT_SOURCE,
			                        PacbaseEditor.CSLINE_COLUMN_TOOL_TIP_DATAELEMENT_KEY];
			
			var grid = new LazyTreeGrid( {
				treeModel : model,
				structure : layoutCSLines,
				height : ( rppCurrentPaneGridHeight ) + "px",
				autoWidth : true,
				selectable : true,
				
				canSort : function () {
					return false;
				},
				onHeaderCellMouseOver  : function (e) {
					dijit.showTooltip(hearderTooltips[e.cellIndex], e.cellNode);
				},
				onHeaderCellMouseOut : function (e) {
					dijit.hideTooltip(e.cellNode);
				} 				
			
			} );
			cpDetailCSLines.addChild( grid );
			return cpDetailCSLines;
		}
	}
} );