/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dijit/registry", "dojox/layout/TableContainer", "dijit/layout/ContentPane", "dijit/form/TextBox", "dijit/form/CheckBox",
		"dijit/layout/BorderContainer", "dojo/keys",
		"dojo/data/ItemFileWriteStore", "dojox/grid/DataGrid", "dijit/form/Textarea", 
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacEnumerationLabel",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( registry, TableContainer, ContentPane, TextBox, CheckBox, BorderContainer, 
				keys, ItemFileWriteStore, DataGrid, Textarea, PacbaseEditor, PacEnumerationLabel, typeDict ) {
	return {
		getSections : function ( data, type, project, streamId, compentId, fileItemId ) {
			var sectionPane;
			require( [
				"dojo/string"
			], function ( string ) {
				sectionPane = new ContentPane( {
					content : "<strong>" + PacbaseEditor.TEXT_SECTION_HEADER + ": </strong><hr><P>"
					+ string.substitute( PacbaseEditor.TEXT_SECTION_DESCRIPTION, [
						typeDict[ type ]
					] ) + "</p>",						
				} );
			} );
						
			
		    var bc = new BorderContainer({
		        style: "height: 90%; width: 100%;"
		    });
		    
		    // create a ContentPane as the left pane in the BorderContainer
		    var cpGrid = new ContentPane({
		        region: "left",
				style : {
					"width" : "100px",
					"height": "auto",
					"overflow" : "auto"
				}
		    });
		    
		    bc.addChild(cpGrid);

		    
			var store = new ItemFileWriteStore( {
				data : {
					items : data.sections
				}
			} );
		
		
			// set up layout
			var layout = [
				{
					'name' : PacbaseEditor.TEXT_SECTION_HEADER,
					'field' : 'sectionCode',
					'width' : '100%',
					'height' : '100%',
				}
				];

			var idx = -1;
			// create a new grid
			var sectionGrid = new DataGrid( {
				store : store,
				structure : layout,
				autoHeight : true,
				selectable : true,
				selectionMode : 'single',
				canSort : function () {
					return false;
				},
				onRowClick : function ( evt ) {
					idx = evt.rowIndex;
					var rowData = this.getItem( evt.rowIndex );
					var sourceId = registry.byId( fileItemId+"_textSourceId" );
					sourceId.set("content", getTextSource(rowData.source[0], streamId, project));
				},
				onKeyDown : function ( evt ) {
					if ( evt.keyCode == keys.DOWN_ARROW || evt.keyCode == keys.TAB ) {
						if ( idx >= sectionGrid.rowCount -1 )
							idx = -1;
						if (sectionGrid.rowCount > 0 && idx < sectionGrid.rowCount) {
							if ( idx == -1 )
								sectionGrid.selection.setSelected(sectionGrid.rowCount -1, false);
							else
								sectionGrid.selection.setSelected(idx, false);
							sectionGrid.selection.setSelected(idx+1, true);
							var rowData = this.getItem( idx + 1 );
							var sourceId = registry.byId( fileItemId+"_textSourceId" );
							sourceId.set("content", getTextSource(rowData.source[0], streamId, project));
							idx++;
						}
					} else if ( evt.keyCode == keys.UP_ARROW ) {
						if ( idx >= sectionGrid.rowCount -1 )
							idx = sectionGrid.rowCount -1;
						if (sectionGrid.rowCount > 0 && idx < sectionGrid.rowCount) {
							if ( idx == 0 ) {
								sectionGrid.selection.setSelected(0, false);
								idx = sectionGrid.rowCount;
							} else {
								sectionGrid.selection.setSelected(idx, false);
							}
							sectionGrid.selection.setSelected(idx - 1, true);
							var rowData = this.getItem( idx - 1 );
							var sourceId = registry.byId( fileItemId+"_textSourceId" );
							sourceId.set("content", getTextSource(rowData.source[0], streamId, project));
							idx--;
						}
					} 
				},	
			} );

			cpGrid.addChild( sectionGrid );
		    

			// create a ContentPane as the center pane in the BorderContainer
			var cpSource = new ContentPane({
				region: "center",
			});
	    
		    bc.addChild(cpSource);
		    
			var sourceTa = new ContentPane( {
				id : fileItemId+'_textSourceId',
				style : {
					"height": "auto",
					"max-height": ( rppCurrentPaneGridHeight ) + "px"
				},
				baseClass: "rppPaneFixedDefaultFont",
			} );
		    cpSource.addChild( sourceTa );
		    
		    sectionPane.addChild(bc)

						
			return sectionPane;
		}
	};
	

	
	function getTextSource ( source , streamId, project ) {
		var newSource  = '';
		var myRegexp = /(@\S+)/g;
		var  idx = 0;
		var match = myRegexp.exec(source);
		while (match != null) {
			newSource += source.substring(idx,match.index).replace(/ /g,"&nbsp;");
			var link = "<a href='javascript:void(0);' onclick='openRadicalElement(&quot;"+streamId+"&quot;, &quot;"+project+"&quot;, &quot;"+project+"&quot;, &quot;"+getLinkType(match[0])+"&quot;, &quot;"+getLinkName(match[0])+"&quot;);return false;'>"+match[0]+"</a>";
			newSource += link;
			idx = match.index + match[0].length;
			match = myRegexp.exec(source);
		}
		newSource += source.substring(idx,source.index).replace(/ /g,"&nbsp;");
		return '<html><body>'+newSource.replace(/\r/g,"<br>").replace(/\n/g,"");
	}
	
	function getLinkType ( link ) {
		var textKey = ["DEL", "DST", "SEG", "TXT", "IAD", "VOL", "DBD", "LIB", "RPT", "OST", "OSV",  "SCR", "SRV",  "OFL", "FLD", "OFV", "FLV", "OFL", "FLD", "OCM", "CMO", "PGM", "MSP", "MET", "USE" ];
		var textTypes = [
			"DataElement", "DataUnit", "DataAggregate", "PacText", "PacInputAid", "PacVolume", "PacBlockBase", "PacLibrary", "PacReport",
			"PacDialog", "PacDialogServer", "PacScreen", "PacServer", "PacDialogFolder", "PacFolder", "PacDialogFolderView", "PacFolderView", 
			"PacDialogFolder", "PacFolder", "PacDialogCommunicationMonitor", "PacCommunicationMonitor", "PacProgram", "PacMacro", "MetaEntity",
			"UserEntity"];
		return textTypes[textKey.indexOf(link.substring(1,4))].toLowerCase();
	}
	
	function getLinkName ( link ) {
		var type = link.substring(1,4);
		var name = link.substring(4);
		if ( "USE" == type ) {
			var idx = name.indexOf(".");
			return name.substring(0,idx);
		} else {
			return name;
		}
	}

	
} );